﻿Public Class AdobeButton
    Inherits Control

    Public Sub New()
        Me.DoubleBuffered = True
        Me.SetStyle(ControlStyles.AllPaintingInWmPaint Or ControlStyles.UserPaint Or _
                    ControlStyles.Opaque Or ControlStyles.OptimizedDoubleBuffer, True)
    End Sub

    Private m_Text As String = String.Empty
    Protected m_State As AdobeButtonState = AdobeButtonState.Default

    Public Overrides Property Text() As String
        Get
            Return m_Text
        End Get
        Set(ByVal value As String)
            m_Text = value
            Me.Invalidate()
        End Set
    End Property

    Protected Overrides Sub OnMouseEnter(ByVal e As System.EventArgs)
        m_State = m_State Or AdobeButtonState.Hover
        Me.Invalidate()
        MyBase.OnMouseEnter(e)
    End Sub

    Protected Overrides Sub OnMouseLeave(ByVal e As System.EventArgs)
        m_State = m_State And Not AdobeButtonState.Hover
        Me.Invalidate()
        MyBase.OnMouseLeave(e)
    End Sub

    Protected Overrides Sub OnGotFocus(ByVal e As System.EventArgs)
        m_State = m_State Or AdobeButtonState.Focus
        Me.Invalidate()
        MyBase.OnGotFocus(e)
    End Sub

    Protected Overrides Sub OnLostFocus(ByVal e As System.EventArgs)
        m_State = m_State And Not AdobeButtonState.Focus
        Me.Invalidate()
        MyBase.OnLostFocus(e)
    End Sub

    Protected Overrides Sub OnMouseDown(ByVal e As System.Windows.Forms.MouseEventArgs)
        m_State = m_State Or AdobeButtonState.Pushed
        Me.Invalidate()
        MyBase.OnMouseDown(e)
    End Sub

    Protected Overrides Sub OnMouseUp(ByVal e As System.Windows.Forms.MouseEventArgs)
        m_State = m_State And Not AdobeButtonState.Pushed
        Me.Invalidate()
        MyBase.OnMouseUp(e)
    End Sub

    Protected Overrides Sub OnKeyDown(ByVal e As System.Windows.Forms.KeyEventArgs)
        If e.KeyCode = Keys.Space Then
            m_State = m_State Or AdobeButtonState.Pushed
            Me.Invalidate()
        End If
        MyBase.OnKeyDown(e)
    End Sub

    Protected Overrides Sub OnKeyUp(ByVal e As System.Windows.Forms.KeyEventArgs)
        If e.KeyCode = Keys.Space Then
            m_State = m_State And Not AdobeButtonState.Pushed
            Me.Invalidate()
            Me.OnClick(EventArgs.Empty)
        End If
        MyBase.OnKeyUp(e)
    End Sub

    Protected Overrides Sub OnPaint(ByVal e As System.Windows.Forms.PaintEventArgs)
        If Me.Enabled Then
            Dim g As Graphics = e.Graphics
            g.TextRenderingHint = Drawing.Text.TextRenderingHint.ClearTypeGridFit
            g.SmoothingMode = Drawing2D.SmoothingMode.AntiAlias

            Dim innerRectangle As Rectangle = Me.ClientRectangle
            'innerRectangle.Inflate(-1, -1)
            innerRectangle.Width -= 1
            innerRectangle.Height -= 1

            If (m_State And AdobeButtonState.Pushed) > 0 Then
                Using backBrush As New SolidBrush(AdobeColors.ButtonPressedColor)
                    g.FillRectangle(backBrush, Me.ClientRectangle)
                End Using
            ElseIf (m_State And AdobeButtonState.Hover) > 0 Then
                Using backBrush As New Drawing2D.LinearGradientBrush(Me.ClientRectangle, AdobeColors.ButtonHighlightGradientColor, AdobeColors.ControlHighlightColor, Drawing2D.LinearGradientMode.Vertical)
                    g.FillRectangle(backBrush, Me.ClientRectangle)
                End Using
            Else
                Using backBrush As New Drawing2D.LinearGradientBrush(Me.ClientRectangle, AdobeColors.ButtonGradientColor, AdobeColors.ControlGenericColor, Drawing2D.LinearGradientMode.Vertical)
                    g.FillRectangle(backBrush, Me.ClientRectangle)
                End Using
            End If

            If (m_State And AdobeButtonState.Focus) > 0 Then
                Using borderPen As New Pen(AdobeColors.ControlFocusColor, 2.0!)
                    g.DrawRectangle(borderPen, innerRectangle)
                End Using
            Else
                Using borderPen As New Pen(AdobeColors.BorderColor)
                    g.DrawRectangle(borderPen, innerRectangle)
                End Using
            End If

            AdobeStringRenderer.DrawText(g, Me.Text, 12, FontStyle.Bold, Me.ClientRectangle, New StringFormat() With {.LineAlignment = StringAlignment.Center, .Alignment = StringAlignment.Center})
        Else
            Using b As New Bitmap(Me.ClientSize.Width, Me.ClientSize.Height, Imaging.PixelFormat.Format24bppRgb)
                Using g As Graphics = Graphics.FromImage(b)
                    g.TextRenderingHint = Drawing.Text.TextRenderingHint.ClearTypeGridFit
                    g.SmoothingMode = Drawing2D.SmoothingMode.AntiAlias

                    Dim innerRectangle As Rectangle = Me.ClientRectangle
                    'innerRectangle.Inflate(-1, -1)
                    innerRectangle.Width -= 1
                    innerRectangle.Height -= 1

                    If (m_State And AdobeButtonState.Pushed) > 0 Then
                        Using backBrush As New SolidBrush(AdobeColors.ButtonPressedColor)
                            g.FillRectangle(backBrush, Me.ClientRectangle)
                        End Using
                    ElseIf (m_State And AdobeButtonState.Hover) > 0 Then
                        Using backBrush As New Drawing2D.LinearGradientBrush(Me.ClientRectangle, AdobeColors.ButtonHighlightGradientColor, AdobeColors.ControlHighlightColor, Drawing2D.LinearGradientMode.Vertical)
                            g.FillRectangle(backBrush, Me.ClientRectangle)
                        End Using
                    Else
                        Using backBrush As New Drawing2D.LinearGradientBrush(Me.ClientRectangle, AdobeColors.ButtonGradientColor, AdobeColors.ControlGenericColor, Drawing2D.LinearGradientMode.Vertical)
                            g.FillRectangle(backBrush, Me.ClientRectangle)
                        End Using
                    End If

                    If (m_State And AdobeButtonState.Focus) > 0 Then
                        Using borderPen As New Pen(AdobeColors.ControlFocusColor, 2.0!)
                            g.DrawRectangle(borderPen, innerRectangle)
                        End Using
                    Else
                        Using borderPen As New Pen(AdobeColors.BorderColor)
                            g.DrawRectangle(borderPen, innerRectangle)
                        End Using
                    End If

                    AdobeStringRenderer.DrawText(g, Me.Text, 12, FontStyle.Bold, Me.ClientRectangle, New StringFormat() With {.LineAlignment = StringAlignment.Center, .Alignment = StringAlignment.Center})
                End Using

                Dim iAttr As New Imaging.ImageAttributes()
                iAttr.SetColorMatrix(New Imaging.ColorMatrix(New Single()() { _
                        New Single() {0.5!, 0.0!, 0.0!, 0.0!, 0.0!}, _
                        New Single() {0.0!, 0.5!, 0.0!, 0.0!, 0.0!}, _
                        New Single() {0.0!, 0.0!, 0.5!, 0.0!, 0.0!}, _
                        New Single() {0.0!, 0.0!, 0.0!, 1.0!, 0.0!}, _
                        New Single() {0.0!, 0.0!, 0.0!, 0.0!, 1.0!} _
                }))

                e.Graphics.DrawImage(b, Me.ClientRectangle, 0, 0, b.Width, b.Height, GraphicsUnit.Pixel, iAttr)
            End Using
        End If
    End Sub
End Class

Public Enum AdobeButtonState
    [Default] = 0
    Focus = 1
    Hover = 2
    Pushed = 4
End Enum