﻿<System.ComponentModel.DefaultEvent("CheckedChanged")> _
Public Class AdobeCheckBox
    Inherits Control

#Region "Events"
    Public Event CheckedChanged As EventHandler

    Protected Overridable Sub OnCheckedChanged(ByVal e As EventArgs)
        RaiseEvent CheckedChanged(Me, e)
    End Sub
#End Region

#Region "Properties"
    Private m_Checked As Boolean = False

    ''' <summary>
    ''' Gets or sets whether or not the <see cref="AdobeCheckBox" /> is checked.
    ''' </summary>
    Public Property Checked() As Boolean
        Get
            Return m_Checked
        End Get
        Set(ByVal value As Boolean)
            m_Checked = value
            Me.Invalidate()
            Me.OnCheckedChanged(EventArgs.Empty)
        End Set
    End Property

    Public Overrides Property Text() As String
        Get
            Return MyBase.Text
        End Get
        Set(ByVal value As String)
            MyBase.Text = value
            Me.Invalidate()
        End Set
    End Property
#End Region

#Region "Constructor"
    Public Sub New()
        Me.DoubleBuffered = True
        Me.SetStyle(ControlStyles.AllPaintingInWmPaint Or ControlStyles.UserPaint Or _
                    ControlStyles.Selectable Or ControlStyles.OptimizedDoubleBuffer Or _
                    ControlStyles.SupportsTransparentBackColor, True)
    End Sub
#End Region

#Region "Drawing"
    Protected Overrides Sub OnPaint(ByVal e As System.Windows.Forms.PaintEventArgs)
        Dim g As Graphics = e.Graphics
        g.SmoothingMode = Drawing2D.SmoothingMode.HighQuality
        g.TextRenderingHint = Drawing.Text.TextRenderingHint.ClearTypeGridFit

        'Draw the box
        Dim boxRect As New Rectangle(5, Me.Height \ 2 - AdobeMetrics.CheckBoxSize \ 2, AdobeMetrics.CheckBoxSize, AdobeMetrics.CheckBoxSize)
        If (m_State And AdobeButtonState.Pushed) > 0 Then
            Using backBrush As New SolidBrush(AdobeColors.ButtonPressedColor)
                g.FillRectangle(backBrush, boxRect)
            End Using
        ElseIf (m_State And AdobeButtonState.Hover) > 0 Then
            Using backBrush As New Drawing2D.LinearGradientBrush(boxRect, AdobeColors.ButtonHighlightGradientColor, AdobeColors.ControlHighlightColor, Drawing2D.LinearGradientMode.Vertical)
                g.FillRectangle(backBrush, boxRect)
            End Using
        Else
            Using backBrush As New Drawing2D.LinearGradientBrush(boxRect, AdobeColors.ButtonGradientColor, AdobeColors.ControlGenericColor, Drawing2D.LinearGradientMode.Vertical)
                g.FillRectangle(backBrush, boxRect)
            End Using
        End If

        'Draw the box's border
        Dim innerbox As Rectangle = boxRect
        innerbox.Inflate(-1, -1)
        Using innerPen As New Pen(AdobeColors.BorderColor)
            g.DrawRectangle(innerPen, innerbox)
        End Using

        'Draw the check mark
        If Me.Checked Then
            Dim center As New Point(innerbox.X + innerbox.Width \ 2, innerbox.Y + innerbox.Height \ 2 + 3)
            Dim cStart As Point = center
            Dim cEnd As Point = center
            cStart.Offset(-4, -5)
            cEnd.Offset(6, -8)
            Using p As New Pen(AdobeColors.ShadowColor, 2.0!)
                g.DrawLines(p, New Point() {cStart, center, cEnd})
            End Using
            center.Offset(-1, -1)
            cStart.Offset(-1, 1)
            cEnd.Offset(-1, -1)
            Using p As New Pen(Color.White, 2.0!)
                g.DrawLines(p, New Point() {cStart, center, cEnd})
            End Using
        End If

        'Draw the text
        Dim s As Size = g.MeasureString(Me.Text, AdobeStringRenderer.DefaultFont(12.0!, FontStyle.Regular)).ToSize()
        AdobeStringRenderer.DrawText(g, Me.Text, 12, FontStyle.Regular, 8 + AdobeMetrics.CheckBoxSize, Me.Height \ 2 - s.Height \ 2)
    End Sub
#End Region

#Region "Mouse Events"
    Protected m_State As AdobeButtonState = AdobeButtonState.Default

    Protected Overrides Sub OnClick(ByVal e As System.EventArgs)
        Me.Checked = Not Me.Checked
        MyBase.OnClick(e)
    End Sub

    Protected Overrides Sub OnDoubleClick(ByVal e As System.EventArgs)
        Me.Checked = Not Me.Checked
        MyBase.OnDoubleClick(e)
    End Sub

    Protected Overrides Sub OnMouseEnter(ByVal e As System.EventArgs)
        m_State = m_State Or AdobeButtonState.Hover
        Me.Invalidate()
        MyBase.OnMouseEnter(e)
    End Sub

    Protected Overrides Sub OnMouseLeave(ByVal e As System.EventArgs)
        m_State = m_State And Not AdobeButtonState.Hover
        Me.Invalidate()
        MyBase.OnMouseLeave(e)
    End Sub

    Protected Overrides Sub OnGotFocus(ByVal e As System.EventArgs)
        m_State = m_State Or AdobeButtonState.Focus
        Me.Invalidate()
        MyBase.OnGotFocus(e)
    End Sub

    Protected Overrides Sub OnLostFocus(ByVal e As System.EventArgs)
        m_State = m_State And Not AdobeButtonState.Focus
        Me.Invalidate()
        MyBase.OnLostFocus(e)
    End Sub

    Protected Overrides Sub OnMouseDown(ByVal e As System.Windows.Forms.MouseEventArgs)
        m_State = m_State Or AdobeButtonState.Pushed
        Me.Invalidate()
        MyBase.OnMouseDown(e)
    End Sub

    Protected Overrides Sub OnMouseUp(ByVal e As System.Windows.Forms.MouseEventArgs)
        m_State = m_State And Not AdobeButtonState.Pushed
        Me.Invalidate()
        MyBase.OnMouseUp(e)
    End Sub

    Protected Overrides Sub OnKeyDown(ByVal e As System.Windows.Forms.KeyEventArgs)
        If e.KeyCode = Keys.Space Then
            m_State = m_State Or AdobeButtonState.Pushed
            Me.Invalidate()
        End If
        MyBase.OnKeyDown(e)
    End Sub

    Protected Overrides Sub OnKeyUp(ByVal e As System.Windows.Forms.KeyEventArgs)
        If e.KeyCode = Keys.Space Then
            m_State = m_State And Not AdobeButtonState.Pushed
            Me.Invalidate()
            Me.OnClick(EventArgs.Empty)
        End If
        MyBase.OnKeyUp(e)
    End Sub
#End Region

End Class