﻿Public Class AdobeForm
    Public Sub New()
        Me.InitializeComponent()
        Me.DoubleBuffered = True
        Me.SetStyle(ControlStyles.AllPaintingInWmPaint Or ControlStyles.UserPaint Or _
                    ControlStyles.OptimizedDoubleBuffer Or ControlStyles.Opaque Or _
                    ControlStyles.ResizeRedraw, True)
    End Sub

    Private m_Header As String = String.Empty
    Private m_ShowHeader As Boolean = True
    Private m_ShowFooter As Boolean = True

    Public Overridable Property Header() As String
        Get
            Return m_Header
        End Get
        Set(ByVal value As String)
            m_Header = value
            Me.Invalidate()
        End Set
    End Property

    Public Overridable Property ShowHeader() As Boolean
        Get
            Return m_ShowHeader
        End Get
        Set(ByVal value As Boolean)
            m_ShowHeader = value
            Me.Invalidate()
        End Set
    End Property

    Public Overridable Property ShowFooter() As Boolean
        Get
            Return m_ShowFooter
        End Get
        Set(ByVal value As Boolean)
            m_ShowFooter = value
            Me.Invalidate()
        End Set
    End Property

    Protected Overrides Sub OnPaint(ByVal e As System.Windows.Forms.PaintEventArgs)
        Dim g As Graphics = e.Graphics
        g.TextRenderingHint = Drawing.Text.TextRenderingHint.AntiAliasGridFit
        g.SmoothingMode = Drawing2D.SmoothingMode.HighQuality
        g.Clear(AdobeColors.BackgroundColor)

        Using borderPen As New Pen(AdobeColors.BorderColor), innerBorderPen As New Pen(AdobeColors.InnerBorderColor)
            'Draw the border
            Dim r As Rectangle = Me.ClientRectangle
            r.Inflate(-1, -1)
            g.DrawRectangle(borderPen, r)

            If Me.ShowHeader Then
                'Draw the header
                Dim headerRect As New Rectangle(0, 0, Me.ClientSize.Width, AdobeStringRenderer.DefaultHeaderSize)
                headerRect.Inflate(-AdobeMetrics.BorderWidth, -AdobeMetrics.BorderWidth)
                Using headerBrush As New SolidBrush(AdobeColors.HeaderBackgroundColor)
                    g.FillRectangle(headerBrush, headerRect)
                    g.DrawLine(borderPen, headerRect.X, headerRect.Bottom, headerRect.Right, headerRect.Bottom)
                    headerRect.Inflate(-AdobeMetrics.BorderWidth, -AdobeMetrics.BorderWidth)
                    g.DrawRectangle(innerBorderPen, headerRect)

                    'Draw the footer
                    If Me.ShowFooter Then
                        Dim footerRect As New Rectangle(0, Me.ClientSize.Height - AdobeStringRenderer.DefaultHeaderSize, Me.ClientSize.Width, AdobeStringRenderer.DefaultHeaderSize)
                        footerRect.Inflate(-AdobeMetrics.BorderWidth, -AdobeMetrics.BorderWidth)
                        g.FillRectangle(headerBrush, footerRect)
                        g.DrawLine(borderPen, footerRect.X, footerRect.Y, footerRect.Right, footerRect.Y)
                        footerRect.Inflate(-AdobeMetrics.BorderWidth, -AdobeMetrics.BorderWidth)
                        g.DrawRectangle(innerBorderPen, footerRect)
                    End If
                End Using

                'Draw the header text
                Dim h As String = If(Me.Header = String.Empty, Me.Text, Me.Header)

                AdobeStringRenderer.DrawText(g, h, AdobeStringRenderer.DefaultHeaderFontSize, FontStyle.Bold, 20, (AdobeStringRenderer.DefaultHeaderSize - AdobeStringRenderer.DefaultHeaderFontSize) \ 2)
            ElseIf Me.ShowFooter Then
                'Draw the footer
                Using headerBrush As New SolidBrush(AdobeColors.HeaderBackgroundColor)
                    Dim footerRect As New Rectangle(0, Me.ClientSize.Height - AdobeStringRenderer.DefaultHeaderSize, Me.ClientSize.Width, AdobeStringRenderer.DefaultHeaderSize)
                    footerRect.Inflate(-AdobeMetrics.BorderWidth, -AdobeMetrics.BorderWidth)
                    g.FillRectangle(headerBrush, footerRect)
                    g.DrawLine(borderPen, footerRect.X, footerRect.Y, footerRect.Right, footerRect.Y)
                    footerRect.Inflate(-AdobeMetrics.BorderWidth, -AdobeMetrics.BorderWidth)
                    g.DrawRectangle(innerBorderPen, footerRect)
                End Using
            End If
        End Using
    End Sub

    'Protected Overrides Sub OnSizeChanged(ByVal e As System.EventArgs)
    '    Me.Invalidate()
    '    MyBase.OnSizeChanged(e)
    'End Sub

    Protected Overrides Sub OnControlAdded(ByVal e As System.Windows.Forms.ControlEventArgs)
        MyBase.OnControlAdded(e)
        If Me.ShowFooter AndAlso e.Control.Top > (Me.ClientSize.Height - AdobeStringRenderer.DefaultHeaderSize) AndAlso e.Control.Anchor = (AnchorStyles.Top Or AnchorStyles.Left) Then _
            e.Control.Anchor = AnchorStyles.Bottom Or If(e.Control.Left + e.Control.Width \ 2 <= Me.ClientSize.Width \ 2, AnchorStyles.Left, AnchorStyles.Right)
    End Sub
End Class
