﻿Imports System.Runtime.CompilerServices

Module Extensions

    ''' <summary>
    ''' Draws an image using 9-slice scaling.
    ''' </summary>
    ''' <param name="g">The <see cref="Graphics" /> object on which to draw the image.</param>
    ''' <param name="img">The <see cref="System.Drawing.Image" /> to draw.</param>
    ''' <param name="dest">The <see cref="System.Drawing.Rectangle" /> in which to draw the image.</param>
    ''' <param name="sliceWidth">The width of a corner.</param>
    ''' <param name="sliceHeight">The height of a corner.</param>
    <Extension()> _
    Public Sub DrawImage9Scale(ByVal g As Graphics, ByVal img As Image, ByVal dest As Rectangle, ByVal sliceWidth As Integer, ByVal sliceHeight As Integer)
        'Draw upper-left
        g.DrawImage(img, New Rectangle(dest.X, dest.Y, sliceWidth, sliceHeight), New Rectangle(0, 0, sliceWidth, sliceHeight), GraphicsUnit.Pixel)
        'Draw lower-left
        g.DrawImage(img, New Rectangle(dest.X, dest.Bottom - sliceHeight, sliceWidth, sliceHeight), New Rectangle(0, img.Height - sliceHeight, sliceWidth, sliceHeight), GraphicsUnit.Pixel)
        'Draw upper-right
        g.DrawImage(img, New Rectangle(dest.Right - sliceWidth, dest.Y, sliceWidth, sliceHeight), New Rectangle(img.Width - sliceWidth, 0, sliceWidth, sliceHeight), GraphicsUnit.Pixel)
        'Draw lower-right
        g.DrawImage(img, New Rectangle(dest.Right - sliceWidth, dest.Bottom - sliceHeight, sliceWidth, sliceHeight), New Rectangle(img.Width - sliceWidth, img.Height - sliceHeight, sliceWidth, sliceHeight), GraphicsUnit.Pixel)

        'Draw left
        g.DrawImage(img, New Rectangle(dest.X, dest.Y + sliceHeight, sliceWidth, dest.Height - sliceHeight * 2), New Rectangle(0, sliceHeight, sliceWidth, img.Height - sliceHeight * 2), GraphicsUnit.Pixel)
        'Draw right
        g.DrawImage(img, New Rectangle(dest.Right - sliceWidth, dest.Y + sliceHeight, sliceWidth, dest.Height - sliceHeight * 2), New Rectangle(img.Width - sliceWidth, sliceHeight, sliceWidth, img.Height - sliceHeight * 2), GraphicsUnit.Pixel)
        'Draw top
        g.DrawImage(img, New Rectangle(dest.X + sliceWidth, dest.Y, dest.Width - sliceWidth * 2, sliceHeight), New Rectangle(sliceWidth, 0, img.Width - sliceWidth * 2, sliceHeight), GraphicsUnit.Pixel)
        'Draw bottom
        g.DrawImage(img, New Rectangle(dest.X + sliceWidth, dest.Bottom - sliceHeight, dest.Width - sliceWidth * 2, sliceHeight), New Rectangle(sliceWidth, img.Height - sliceHeight, img.Width - sliceWidth * 2, sliceHeight), GraphicsUnit.Pixel)

        'Draw inner
        g.DrawImage(img, New Rectangle(dest.X + sliceWidth, dest.Y + sliceHeight, dest.Width - sliceWidth * 2, dest.Height - sliceHeight * 2), New Rectangle(sliceWidth, sliceHeight, img.Width - sliceWidth * 2, img.Height - sliceHeight * 2), GraphicsUnit.Pixel)
    End Sub

    ''' <summary>
    ''' Draws an image using 9-slice scaling.
    ''' </summary>
    ''' <param name="g">The <see cref="Graphics" /> object on which to draw the image.</param>
    ''' <param name="img">The <see cref="System.Drawing.Image" /> to draw.</param>
    ''' <param name="dest">The <see cref="System.Drawing.Rectangle" /> in which to draw the image.</param>
    ''' <param name="sliceWidth">The width of a corner.</param>
    ''' <param name="sliceHeight">The height of a corner.</param>
    ''' <param name="alpha">The opacity at which to draw the image.</param>
    <Extension()> _
    Public Sub DrawImage9Scale(ByVal g As Graphics, ByVal img As Image, ByVal dest As Rectangle, ByVal sliceWidth As Integer, ByVal sliceHeight As Integer, ByVal alpha As Single)
        Using b As New Bitmap(dest.Width, dest.Height)

            Using bg As Graphics = Graphics.FromImage(b)
                bg.DrawImage9Scale(img, New Rectangle(0, 0, dest.Width, dest.Height), sliceWidth, sliceHeight)
            End Using

            Dim iAttr As New Imaging.ImageAttributes()
            iAttr.SetColorMatrix(New Imaging.ColorMatrix(New Single()() { _
                New Single() {1.0!, 0.0!, 0.0!, 0.0!, 0.0!}, _
                New Single() {0.0!, 1.0!, 0.0!, 0.0!, 0.0!}, _
                New Single() {0.0!, 0.0!, 1.0!, 0.0!, 0.0!}, _
                New Single() {0.0!, 0.0!, 0.0!, alpha, 0.0!}, _
                New Single() {0.0!, 0.0!, 0.0!, 0.0!, 1.0!} _
            }))

            g.DrawImage(b, dest, 0, 0, b.Width, b.Height, GraphicsUnit.Pixel, iAttr)
        End Using

    End Sub

    <Extension()> _
    Public Function Round(ByVal r As Rectangle, ByVal radius As Integer) As Drawing2D.GraphicsPath
        Dim gp As New Drawing2D.GraphicsPath()
        gp.AddArc(New Rectangle(r.X, r.Y, radius, radius), 180.0!, 90.0!)
        gp.AddArc(New Rectangle(r.Right - radius, r.Y, radius, radius), -90.0!, 90.0!)
        gp.AddArc(New Rectangle(r.Right - radius, r.Bottom - radius, radius, radius), 0.0!, 90.0!)
        gp.AddArc(New Rectangle(r.X, r.Bottom - radius, radius, radius), 90.0!, 90.0!)
        gp.CloseFigure()
        Return gp
    End Function

    <Extension()> _
    Public Sub DrawRoundRectangle(ByVal g As Graphics, ByVal p As Pen, ByVal x As Integer, ByVal y As Integer, ByVal width As Integer, ByVal height As Integer, ByVal radius As Integer)
        g.DrawRoundRectangle(p, New Rectangle(x, y, width, height), radius)
    End Sub

    <Extension()> _
    Public Sub DrawRoundRectangle(ByVal g As Graphics, ByVal p As Pen, ByVal r As Rectangle, ByVal radius As Integer)
        Using gp As Drawing2D.GraphicsPath = r.Round(radius)
            g.DrawPath(p, gp)
        End Using
    End Sub

    <Extension()> _
    Public Sub FillRoundRectangle(ByVal g As Graphics, ByVal b As Brush, ByVal x As Integer, ByVal y As Integer, ByVal width As Integer, ByVal height As Integer, ByVal radius As Integer)
        g.FillRoundRectangle(b, New Rectangle(x, y, width, height), radius)
    End Sub

    <Extension()> _
    Public Sub FillRoundRectangle(ByVal g As Graphics, ByVal b As Brush, ByVal r As Rectangle, ByVal radius As Integer)
        Using gp As Drawing2D.GraphicsPath = r.Round(radius)
            g.FillPath(b, gp)
        End Using
    End Sub

End Module
